#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#Copyright (c) 2006 - 2007 by The WordForge Foundation
#                       www.wordforgefoundation.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# See the LICENSE file for more details. 
#
# Developed by:
#       Hok Kakada (hokkakada@khmeros.info)
#       Keo Sophon (keosophon@khmeros.info)
#       San Titvirak (titvirak@khmeros.info)
#       Seth Chanratha (sethchanratha@khmeros.info)
# 
# This module stores status

import WordForge.Editor.modules.World as World

class Status:
    
    def __init__(self, store):
        """
        Set the number of translated, fuzzy, untranslated units to global variables.
        
        @param store: the new storage class.
        """        
        self.__numTranslated__ = 0
        self.__numFuzzy__ = 0
        self.__numUntranslated__ = 0
        self.__numTotal__ = 0
        self.__nplurals__ = World.settings.value("nPlural").toInt()[0]
    
    def unitState(self, unit):
        """
        return bitwise indicating state of unit.
        bitwises are defined in World class.
        
        @param unit: a class unit whose state is returned.
        """
        state = 0
        if (unit.isheader()):
            return state
        if (unit.isfuzzy()):
            state = World.fuzzy
            self.__numFuzzy__ +=1
        elif (unit.istranslated()):
            state = World.translated
            self.__numTranslated__ +=1
        else:
            state = World.untranslated
            self.__numUntranslated__ +=1
        
        if (unit.hasplural() and (self.__nplurals__ > 1)):
            state += World.plural
        
        return state
        
    def markFuzzy(self, unit, fuzzy):
        """
        toggle fuzzy status of a given unit.
        
        @param unit: a class unit whose fuzzy status is toggled.
        @param fuzzy: type as bool.
        """
        # in case unit is already fuzzy, so we don't need to set it is fuzzy again.
        if (unit.isfuzzy() and fuzzy):
            return
        
        # unit can be set fuzzy from toggling translated unit
        # or unit can be set fuzzy from untranslated unit with autotranslation
        unitStatusBefore = unit.x_editor_state
        
        # mark unit to fuzzy true or false
        unit.markfuzzy(fuzzy)
        
        if (fuzzy):
            self.__numFuzzy__ += 1
            unit.x_editor_state |= World.fuzzy            
            # in case toggle from translated unit
            if (unitStatusBefore ==  2):
                self.__numTranslated__ -= 1
                unit.x_editor_state &= ~World.translated
            else:
            # in case autotranslate from untranslated unit
                self.__numUntranslated__ -=1
                unit.x_editor_state &= ~World.untranslated
        else:
            # it can be set fuzzy to false from toggling translated unit
            if self.__numFuzzy__ >0:
                self.__numFuzzy__ -= 1
                self.__numTranslated__ += 1
            unit.x_editor_state &= ~World.fuzzy
            unit.x_editor_state |= World.translated

    def markTranslated(self, unit, bool):
        """
        toggle translated status of a given unit.
        
        @param unit a class unit whose translated status is toggled.
        @param bool True or False
        """
        if (unit.isfuzzy()):
            self.markFuzzy(unit, False)
        if (bool):
            if (unit.x_editor_state & World.translated):
                return 
            self.__numTranslated__ += 1
            self.__numUntranslated__ -= 1
            unit.x_editor_state |= World.translated
            unit.x_editor_state &= ~World.untranslated
        else:
            if (unit.x_editor_state & World.untranslated):                
                return   
            self.__numTranslated__ -= 1
            self.__numUntranslated__ += 1
            unit.x_editor_state &= ~World.translated
            unit.x_editor_state |= World.untranslated
    def getTotalOfStrings(self):
        """
        @ return total of strings
        """
        return self.__numTranslated__ + self.__numUntranslated__ + self.__numFuzzy__
        
    def getTotalOfTranslatedStrings(self):
        """
        @return tolal of translated strings
        """
        return self.__numTranslated__
        
    def getTotalOfUntranslatedStrings(self):
        """
        @return total of untranslated strings
        """
        return self.__numUntranslated__
        
    def getTotalOfFuzzyStrings(self):
        """
        @return toal of fuzzy strings
        """
        return self.__numFuzzy__
        

