#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2006-2007 Zuza Software Foundation
# 
# This file is part of translate.
#
# translate is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# translate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with translate; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""module for handling TBX glossary files"""

from WordForge.translate.storage import lisa
from lxml import etree

XML_NS = 'http://www.w3.org/XML/1998/namespace'

class tbxunit(lisa.LISAunit):
    """A single term in the TBX file. 
Provisional work is done to make several languages possible."""
    rootNode = "termEntry"
    languageNode = "langSet"
    textNode = "term"
                   
    def createlanguageNode(self, lang, text, purpose):
        """returns a langset xml Element setup with given parameters"""
        if isinstance(text, str):
            text = text.decode("utf-8")
        langset = etree.Element(self.languageNode)
        lisa.setXMLlang(langset, lang)
        tig = etree.SubElement(langset, "tig") # or ntig with termGrp inside
        term = etree.SubElement(tig, self.textNode)
        term.text = text
        return langset
        
    def elementToData(self, element):
        XML_NS = 'http://www.w3.org/XML/1998/namespace'
#      This function takes the element and parses the data and put them in appropiate data structure or data type.
#      Holds the definition at termentry level
        self.entryDefinition = []
#      Holds the subjectField
        self.subjectField = []
#      Holds langSet level definition.This dictionary contains lists inside.
        self.languageDependentDefinition = {}
#      Holds the target terms 
        self.targets = []
#      Holds the POS of target terms.
        self.targetPOSDict = {}
#      Holds the target contexts.
        self.context = []
#      Find termentry level definition and subject field.
        for eachtag in element.iter():
            if eachtag.tag =='langSet':
                break
            if eachtag.tag =='descrip': 
                if eachtag.get('type') == 'definition':
                    self.entryDefinition.append(eachtag.text)
                elif eachtag.get('type') == 'subjectField':
                    self.subjectField.append(eachtag.text)
#                elif eachtag.get('type') == 'context':
#                    self.context.append(eachtag.text)
#      Find language level definition, POS and Context
        langSetList =  element.findall('langSet')
        for eachLangSet in langSetList:
            langDescripList = eachLangSet.findall('descrip')
            langLevelDefinition = []
            for eachDescrip in langDescripList:
                if eachDescrip.get('type') == 'definition':
                    langLevelDefinition.append(eachDescrip.text)
            langName = eachLangSet.get("{%s}lang" % XML_NS)
            self.languageDependentDefinition[langName] = langLevelDefinition
            
        for i in range (0, len(langSetList)):
#          Find all the tig tagged elements.
            tigList = langSetList[i].findall('tig')
#          If this is the source term them, read the source context
            if i==0 and tigList:            
                descripList = tigList[0].findall('descrip')
                for each in descripList:
                    if each.get('type')=='context':
                        self.context.append(each.text)
                            
#          If this is the target tag then snatch information from it.
            if i>0 and len(tigList)>0:
                for eachTig in tigList:
                    termText = eachTig.find('term').text
                    self.targets.append(termText) 
                    if termText is None:
                        self.targetPOSDict[''] = ''
                    else:
                        self.targetPOSDict[termText] = ''
                    termNoteList = eachTig.findall('termNote')
                    for eachTermNote in termNoteList:
                        if eachTermNote.get('type') == 'partOfSpeech':
                            if termText is None:
                                self.targetPOSDict[''] = eachTermNote.text
                            else:
                                self.targetPOSDict[termText] = eachTermNote.text                   
                            
    def getTbxAttrFromCsv(self, csvunit):
#        This function takes the attributes from csvunit and puts them in tbxunit. This function is used in csv2tbx converter.

        self.subject = csvunit.subject
        self.definition = csvunit.definition
        self.context = csvunit.context
        self.part_of_speech = csvunit.part_of_speech
        self.source_definition = csvunit.source_definition
        self.target_part_of_speech = csvunit.target_part_of_speech
        self.target_definition = csvunit.target_definition
    
    def setInfoInTbx(self, tbxunit, targetlanguage = None):
#      The task of this function is to put in the extra bit of glossary information in tbxunit's xmlelement attribute.

        if tbxunit.definition:
            tbxunit.xmlelement.insert(0, self.setElement("descrip", "type", "definition" , tbxunit.definition))
        if tbxunit.subject:
            tbxunit.xmlelement.insert(0, self.setElement("descrip", "type", "subjectField" , tbxunit.subject))
            
        langsetList =  tbxunit.xmlelement.findall('langSet')
        
        if targetlanguage:
            if len(langsetList)>1:
                langsetList[1].set("{%s}lang" % XML_NS, targetlanguage)
         
        if tbxunit.source_definition:
            if langsetList:
                langsetList[0].insert(0, self.setElement("descrip", "type", "definition" , tbxunit.source_definition))
                
        if tbxunit.part_of_speech:
            if langsetList:
                tig = langsetList[0].find('tig')
                if tig is not None:
                    tig.append(self.setElement("termNote", "type", "partOfSpeech" , tbxunit.part_of_speech))
         
        if tbxunit.context:
            if langsetList:
                tig = langsetList[0].find('tig')
                if tig is not None:
                    tig.append(self.setElement("descrip", "type", "context" , tbxunit.context))
                    
        if tbxunit.target: 
            targets = tbxunit.target.split(',')
#          Remove the existing tig if more than one target  
            if len(targets)>1:
                if len(langsetList)>1:
                    tig = langsetList[1].find('tig')
                    if tig is not None:
                        langsetList[1].remove(tig)
                    for each in reversed(targets):
                        tig = etree.Element('tig')
                        tig.insert(0, self.setElement('term',None, None, each ))
                        langsetList[1].insert(0, tig)
        
        if tbxunit.target_part_of_speech:
            pos = tbxunit.target_part_of_speech.split(',')
            if len(langsetList)>1:
                tigList = langsetList[1].findall('tig')
            for i in range(len(pos)):
                if i<len(tigList):
                    tig = tigList[i]
                    tig.append(self.setElement("termNote", "type", "partOfSpeech" , pos[i]))
                        
        if tbxunit.target_definition:
            if len(langsetList)>1:
                langsetList[1].insert(0, self.setElement("descrip", "type", "definition" , tbxunit.target_definition))
                        
    def setElement(self, tagname, attrname, attrvalue, text):
#  This function returns an element with the specified values.
        element = etree.Element(tagname)
        if attrname and attrvalue:
            element.set(attrname, attrvalue)
        element.text = text
        return element

    def po2tbxmap(self, tbxunit, pounit=None):
#   This function copies some pounit data into the tbxunit 

        tbxunit.definition  = None
        tbxunit.subject  = None
        tbxunit.source_definition = None
        tbxunit.part_of_speech = None
        tbxunit.context  = None
        tbxunit.target_part_of_speech = None
        tbxunit.target_definition = None
        
        if pounit:
            if pounit.entryDefinition:
                tbxunit.definition  = pounit.entryDefinition[0]
            if pounit.subjectField:
                tbxunit.subject  = pounit.subjectField[0]     
            if pounit.languageDependentDefinition: 
                if pounit.languageDependentDefinition.get('en'):
                    tbxunit.source_definition = ','.join(pounit.languageDependentDefinition.get('en'))
                for langname in pounit.languageDependentDefinition.keys():
                    if langname != 'en':
                        tbxunit.target_definition = ','.join(pounit.languageDependentDefinition.get(langname))
            if pounit.context:
                tbxunit.context  = pounit.context[0]
            if pounit.targetPOSDict:
                tbxunit.target_part_of_speech = ','.join([pounit.targetPOSDict[k] for k in pounit.targetPOSDict.keys()])

class tbxfile(lisa.LISAfile):
    """Class representing a TBX file store."""
    UnitClass = tbxunit
    Name = _("TBX file")
    Mimetypes  = ["application/x-tbx"]
    Extensions = ["tbx"]
    rootNode = "martif"
    bodyNode = "body"
    XMLskeleton = '''<?xml version="1.0"?>
<!DOCTYPE martif PUBLIC "ISO 12200:1999A//DTD MARTIF core (DXFcdV04)//EN" "TBXcdv04.dtd">
<martif type="TBX">
<martifHeader>
<fileDesc>
<sourceDesc><p>Translate Toolkit - csv2tbx</p></sourceDesc>
</fileDesc>
</martifHeader>
<text><body></body></text>
</martif>'''
    
    def __init__(self, *args, **kwargs):
        lisa.LISAfile.__init__(self, *args, **kwargs)
        self._filename = "NoName"
        self._messagenum = 0
        
    def initbody(self):
        """Initialises self.body so it never needs to be retrieved from the XML again."""
        self.namespace = self.document.getroot().nsmap.get(None, None)
        self.body = self.document.find('//%s' % self.namespaced(self.bodyNode))
#        self.body = self.document.getroot()[0].find(self.namespaced(self.bodyNode))
    
    def addheader(self):
        """Initialise headers with TBX specific things."""
        lisa.setXMLlang(self.document.getroot(), self.sourcelanguage)

    def parsestring(cls, storestring):
        """Parses the string to return the correct file object"""
        tbx = super(tbxfile, cls).parsestring(storestring)
        return tbx
    parsestring = classmethod(parsestring)

    def switchfile(self, filename, createifmissing=False):
        """adds the given trans-unit (will create the nodes required if asked). Returns success"""
        self._filename = filename
        filenode = self.getfilenode("NoName")
        if filenode:
            self.document.getroot().remove(filenode)
        filenode = self.getfilenode(filename)
        if filenode is None:
            if not createifmissing:
                return False
            filenode = self.createfilenode(filename)
            self.document.getroot().append(filenode)

        self.body = self.getbodynode(filenode, createifmissing=createifmissing)
        if self.body is None:
            return False
        self._messagenum = len(list(self.body.iterdescendants(self.namespaced("trans-unit"))))
        #TODO: was 0 based before - consider
    #    messagenum = len(self.units)
        #TODO: we want to number them consecutively inside a body/file tag
        #instead of globally in the whole XLIFF file, but using len(self.units)
        #will be much faster
        return True
        
    def addsourceunitWF(self, source, filename="NoName", createifmissing=False):
        """adds the given trans-unit to the last used body node if the filename has changed it uses the slow method instead (will create the nodes required if asked). Returns success"""
        if self._filename != filename:
            if not self.switchfile(filename, createifmissing):
                return None
        unit = super(tbxfile, self).addsourceunit(source)
        self._messagenum += 1
        return unit
